/**
 * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef NETWORKPACKETTYPES_H
#define NETWORKPACKETTYPES_H

#define PACKET_TYPE_IDENTITY QStringLiteral("kdeconnect.identity")
#define PACKET_TYPE_PAIR QStringLiteral("kdeconnect.pair")

#endif // NETWORKPACKETTYPES_H
