/**
 * SPDX-FileCopyrightText: 2013 Albert Vaca <albertvaka@gmail.com>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef CORE_DEBUG_H
#define CORE_DEBUG_H

#include "kdeconnectcore_export.h"
#include "kdeconnect_debug.h"

void logBacktrace();

#endif //CORE_DEBUG_H
