/*
 * SPDX-FileCopyrightText: 2020 Weixuan XIAO <veyx.shaw@gmail.com>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef SERVICE_REGISTER_H
#define SERVICE_REGISTER_H

void registerServices();

#endif
