/**
 * SPDX-FileCopyrightText: 2020 Jiří Wolker <woljiri@gmail.com>
 *
 * SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 */

#ifndef GSMASCIIMAP_H
#define GSMASCIIMAP_H

extern bool gsm_ascii_map[128];

#endif // GSMASCIIMAP_H

